@@ -1,25 +1,28 @@
 # -*- coding: UTF-8 -*-
-# Wammu - Phone manager
-# Copyright (c) 2003 - 2006 Michal Čihař
-#
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published by
-# the Free Software Foundation.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
-# more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 '''
+Wammu - Phone manager
 SMS composer
 '''
+__author__ = 'Michal Čihař'
+__email__ = 'michal@cihar.com'
+__license__ = '''
+Copyright (c) 2003 - 2006 Michal Čihař
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License version 2 as published by
+the Free Software Foundation.
+
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+'''
 
 import wx
-import wx.lib.rcsizer
 import wx.lib.editor.editor
 import wx.lib.mixins.listctrl
 import Wammu
@@ -68,7 +71,7 @@ class StyleEdit(wx.Dialog):
     def __init__(self, parent, entry):
         wx.Dialog.__init__(self, parent, -1, _('Text style'))
 
-        self.sizer = wx.lib.rcsizer.RowColSizer()
+        self.sizer = wx.GridBagSizer()
 
         self.entry = entry
 
@@ -117,10 +120,10 @@ class StyleEdit(wx.Dialog):
             col = 1
 
         self.ok = wx.Button(self, wx.ID_OK, _('OK'))
-        self.sizer.Add(self.ok, pos = (row, 1), colspan = maxcol, flag = wx.ALIGN_CENTER)
+        self.sizer.Add(self.ok, pos = (row, 1), span = wx.GBSpan(colspan = maxcol), flag = wx.ALIGN_CENTER)
         wx.EVT_BUTTON(self, wx.ID_OK, self.Okay)
 
-        self.sizer.AddSpacer(5,5, pos=(row + 1, maxcol + 1))
+        self.sizer.AddSpacer((5,5), pos=(row + 1, maxcol + 1))
 
         self.sizer.Fit(self)
         self.SetAutoLayout(True)
@@ -156,11 +159,11 @@ class TextEditor(GenericEditor):
 
         self.backuptext = ''
 
-        self.sizer = wx.lib.rcsizer.RowColSizer()
+        self.sizer = wx.GridBagSizer()
 
         self.edit = wx.TextCtrl(self, -1, style=wx.TE_MULTILINE)
 
-        self.sizer.Add(self.edit, pos = (0,0), flag = wx.EXPAND, colspan = 4)
+        self.sizer.Add(self.edit, pos = (0,0), flag = wx.EXPAND, span = wx.GBSpan(colspan = 4))
         self.sizer.AddGrowableCol(1)
         self.sizer.AddGrowableCol(2)
         self.sizer.AddGrowableRow(0)
@@ -231,7 +234,7 @@ class TextEditor(GenericEditor):
 class PredefinedAnimEditor(GenericEditor):
     def __init__(self, parent, part, cfg, unicode):
         GenericEditor.__init__(self, parent, part, cfg, unicode)
-        self.sizer = wx.lib.rcsizer.RowColSizer()
+        self.sizer = wx.GridBagSizer()
 
         values = []
         for x in Wammu.Data.PredefinedAnimations:
@@ -254,7 +257,7 @@ class PredefinedAnimEditor(GenericEditor):
         self.edit.SetSelection(self.part['Number'])
         self.OnChange()
 
-        self.sizer.Add(wx.StaticText(self, -1, _('Select predefined animation:')), pos = (0,0), flag = wx.ALIGN_CENTER_VERTICAL)
+        self.sizer.Add(wx.StaticText(self, -1, _('Select predefined animation:')), pos = (0,0), flag = wx.ALIGN_CENTER_VERTICAL | wx.EXPAND)
         self.sizer.Add(self.edit, pos = (0,1), flag = wx.ALIGN_CENTER)
         self.sizer.Add(self.bitmap, pos = (0,2), flag = wx.ALIGN_CENTER)
 
@@ -274,7 +277,7 @@ class PredefinedAnimEditor(GenericEditor):
 class PredefinedSoundEditor(GenericEditor):
     def __init__(self, parent, part, cfg, unicode):
         GenericEditor.__init__(self, parent, part, cfg, unicode)
-        self.sizer = wx.lib.rcsizer.RowColSizer()
+        self.sizer = wx.GridBagSizer()
 
         values = []
         for x in Wammu.Data.PredefinedSounds:
@@ -333,7 +336,7 @@ class SMSComposer(wx.Dialog):
         if not entry.has_key('Number'):
             entry['Number'] = ''
 
-        self.sizer = wx.lib.rcsizer.RowColSizer()
+        self.sizer = wx.GridBagSizer()
 
 
         row = 1
@@ -365,7 +368,7 @@ class SMSComposer(wx.Dialog):
         self.contbut.SetToolTipString(_('Add number of recipient from contacts.'))
 
         self.sizer.Add(wx.StaticText(self, -1, _('Recipient\'s numbers:')), pos = (row,1), flag = wx.ALIGN_LEFT)
-        self.sizer.Add(self.number, pos = (row,2), flag = wx.EXPAND, colspan = 5)
+        self.sizer.Add(self.number, pos = (row,2), flag = wx.EXPAND, span = wx.GBSpan(colspan = 5))
         self.sizer.Add(self.contbut, pos = (row,7), flag = wx.ALIGN_CENTER)
 
         wx.EVT_BUTTON(self, self.contbut.GetId(), self.ContactPressed)
@@ -385,6 +388,7 @@ class SMSComposer(wx.Dialog):
 
         self.report = wx.CheckBox(self, -1, _('Delivery report'))
         self.report.SetToolTipString(_('Check to request delivery report for message.'))
+        self.report.SetValue(self.cfg.Read('/Message/DeliveryReport', 'no') == 'yes')
         self.sizer.Add(self.report, pos = (row,2), flag = wx.ALIGN_LEFT)
 
         self.sent = wx.CheckBox(self, -1, _('Sent'))
@@ -408,10 +412,10 @@ class SMSComposer(wx.Dialog):
         wx.EVT_BUTTON(self, self.addbut.GetId(), self.AddPressed)
         wx.EVT_BUTTON(self, self.delbut.GetId(), self.DeletePressed)
 
-        self.sizer.Add(self.current, pos = (row,1), flag = wx.EXPAND, colspan = 2, rowspan = 2)
+        self.sizer.Add(self.current, pos = (row,1), flag = wx.EXPAND, span = wx.GBSpan(colspan = 2, rowspan = 2))
         self.sizer.Add(self.addbut, pos = (row,4), flag = wx.ALIGN_CENTER)
         self.sizer.Add(self.delbut, pos = (row + 1,4), flag = wx.ALIGN_CENTER)
-        self.sizer.Add(self.available, pos = (row,6), flag = wx.EXPAND, colspan = 2, rowspan = 2)
+        self.sizer.Add(self.available, pos = (row,6), flag = wx.EXPAND, span = wx.GBSpan(colspan = 2, rowspan = 2))
 
         row = row + 3
 
@@ -429,7 +433,7 @@ class SMSComposer(wx.Dialog):
         self.editorrow = row
 
         self.editor = wx.StaticText(self, -1, _('Create new message by adding part to left list...'), size = (-1, 150))
-        self.sizer.Add(self.editor, pos = (row,1), flag = wx.EXPAND, colspan = 7)
+        self.sizer.Add(self.editor, pos = (row,1), flag = wx.EXPAND, span = wx.GBSpan(colspan = 7))
 
         row = row + 2
 
@@ -445,7 +449,7 @@ class SMSComposer(wx.Dialog):
         wx.EVT_BUTTON(self, wx.ID_OK, self.Okay)
         wx.EVT_BUTTON(self, self.preview.GetId(), self.Preview)
 
-        self.sizer.AddSpacer(5,5, pos=(row + 1,8))
+        self.sizer.AddSpacer((5,5), pos=(row + 1,8))
         self.sizer.AddGrowableCol(1)
         self.sizer.AddGrowableCol(2)
         self.sizer.AddGrowableCol(6)
@@ -503,6 +507,14 @@ class SMSComposer(wx.Dialog):
             while select > count:
                 select = select - 1
             self.current.SetItemState(select, wx.LIST_STATE_FOCUSED | wx.LIST_STATE_SELECTED, wx.LIST_STATE_FOCUSED | wx.LIST_STATE_SELECTED)
+        else:
+            if hasattr(self, 'editor'):
+                self.sizer.Detach(self.editor)
+                self.editor.Destroy()
+                del self.editor
+            self.editor = wx.StaticText(self, -1, _('Create new message by adding part to left list...'), size = (-1, 150))
+            self.sizer.Add(self.editor, pos = (self.editorrow,1), flag = wx.EXPAND, span = wx.GBSpan(colspan = 7))
+            self.sizer.Layout()
 
     def AvailableSelected(self, event):
         self.availsel = event.m_itemIndex
@@ -515,21 +527,24 @@ class SMSComposer(wx.Dialog):
     def CurrentSelected(self, event):
         self.StoreEdited()
         if hasattr(self, 'editor'):
+            self.sizer.Detach(self.editor)
+            self.editor.Destroy()
             del self.editor
 
         found = False
         for p in SMSParts:
             if self.entry['SMSInfo']['Entries'][event.m_itemIndex]['ID'] in p[2]:
                 self.editor = p[3](self, self.entry['SMSInfo']['Entries'][event.m_itemIndex], self.cfg, self.unicode.GetValue())
-                self.sizer.Add(self.editor, pos = (self.editorrow,1), flag = wx.EXPAND, colspan = 7)
+                self.sizer.Add(self.editor, pos = (self.editorrow,1), flag = wx.EXPAND, span = wx.GBSpan(colspan = 7))
                 found = True
                 break
         if not found:
             self.editor = wx.StaticText(self, -1, _('No editor available for type %s') % self.entry['SMSInfo']['Entries'][event.m_itemIndex]['ID'])
-            self.sizer.Add(self.editor, pos = (self.editorrow,1), flag = wx.EXPAND, colspan = 7)
+            self.sizer.Add(self.editor, pos = (self.editorrow,1), flag = wx.EXPAND, span = wx.GBSpan(colspan = 7))
             self.prevedit = -1
         else:
             self.prevedit = event.m_itemIndex
+        self.sizer.Layout()
 
     def UpPressed(self, evt):
         if self.prevedit == -1:
@@ -567,7 +582,7 @@ class SMSComposer(wx.Dialog):
         self.GenerateCurrent(max(next, 0))
 
     def AddPressed(self, evt):
-        if self.availsel == -1 or self.prevedit == -1:
+        if self.availsel == -1:
             return
         v = {'ID': SMSParts[self.availsel][4]}
         if v['ID'][-5:] == '16bit' and self.cfg.Read('/Message/16bitId', 'yes') != 'yes':
