@@ -1,22 +1,26 @@
 # -*- coding: UTF-8 -*-
-# Wammu - Phone manager
-# Copyright (c) 2003 - 2006 Michal Čihař
-#
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published by
-# the Free Software Foundation.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
-# more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 '''
+Wammu - Phone manager
 Unexpected exception handler
 '''
+__author__ = 'Michal Čihař'
+__email__ = 'michal@cihar.com'
+__license__ = '''
+Copyright (c) 2003 - 2006 Michal Čihař
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License version 2 as published by
+the Free Software Foundation.
+
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+'''
 
 import Wammu
 if Wammu.gammu_error == None:
@@ -53,19 +57,20 @@ def Handler(type, value, tback):
     textexc = ''.join(traceback.format_exception_only(type, value))
 
     # debug log information
-    if debuglogfilename is not None:
-        # copy debug log
-        handle, name = tempfile.mkstemp('.log', 'wammu-crash-')
-        outf = os.fdopen(handle, 'w+')
-        inf = open(debuglogfilename, 'r')
-        outf.write(inf.read())
-        outf.close()
-        inf.close()
-        print 'Created debug log copy in %s for error reporting.' % name
+    logtext = ''
+    try:
+        if debuglogfilename is not None:
+            # copy debug log
+            handle, name = tempfile.mkstemp('.log', 'wammu-crash-')
+            outf = os.fdopen(handle, 'w+')
+            inf = open(debuglogfilename, 'r')
+            outf.write(inf.read())
+            inf.close()
+            print 'Created debug log copy in %s for error reporting.' % name
 
-        logtext =  '\n%s\n' % _('Debug log was saved for phone communication, if this error appeared during communicating with phone, you are strongly encouraged to include it in bugreport. Debug log is saved in file %s.') % name
-    else:
-        logtext = ''
+            logtext =  '\n%s\n' % _('Debug log was saved for phone communication, if this error appeared during communicating with phone, you are strongly encouraged to include it in bugreport. Debug log is saved in file %s.') % name
+    except:
+        pass
 
     # traceback id (md5 sum of last topmost traceback item inside Wammu - file(function):code)
     try:
@@ -85,17 +90,30 @@ def Handler(type, value, tback):
     else:
         unicodewarning = ''
 
+    bluez = 'None'
+    try:
+        import bluetooth
+        bluez = 'PyBluez'
+    except ImportError:
+        try:
+            import btctl
+            bluez = 'btctl'
+        except ImportError:
+            pass
+
     # prepare message
     text = """%s
 
 %s
 %s%s%s
 --------------- System information ----------------
+Platform     %s
 Python       %s
 wxPython     %s
 Wammu        %s
 python-gammu %s
 Gammu        %s
+Bluetooth    %s
 locales      %s (%s)
 ------------------ Traceback ID -------------------
 %s
@@ -105,7 +123,15 @@ locales      %s (%s)
 """ % (
     _('Unhandled exception appeared.'),
     _('If you want to help improving this program, please submit following infomation and description how did it happen to %s. Please report in english, otherwise you will be most likely told to translate you report to english later.') % 'http://bugs.cihar.com',
-    logtext, tracetext, unicodewarning, pyver, wxver, wammuver, pgammuver, gammuver, loc, charset, traceid, texttrace, textexc)
+    logtext, tracetext, unicodewarning, sys.platform, pyver, wxver, wammuver, pgammuver, gammuver, bluez, loc, charset, traceid, texttrace, textexc)
+
+    # Include exception info in crash file
+    try:
+        if debuglogfilename is not None:
+            outf.write(text.encode('utf-8'))
+            outf.close()
+    except:
+        pass
 
     # display error
     try:
