@@ -1,22 +1,26 @@
 # -*- coding: UTF-8 -*-
-# Wammu - Phone manager
-# Copyright (c) 2003 - 2006 Michal Čihař
-#
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published by
-# the Free Software Foundation.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
-# more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 '''
+Wammu - Phone manager
 Main Wammu window
 '''
+__author__ = 'Michal Čihař'
+__email__ = 'michal@cihar.com'
+__license__ = '''
+Copyright (c) 2003 - 2006 Michal Čihař
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License version 2 as published by
+the Free Software Foundation.
+
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+'''
 
 import wx
 import wx.html
@@ -131,7 +135,10 @@ class WammuFrame(wx.Frame):
         wx.Frame.__init__(self, parent, id, 'Wammu', pos, size, wx.DEFAULT_FRAME_STYLE)
 
         icon = wx.EmptyIcon()
-        icon.CopyFromBitmap(wx.Bitmap(AppIconPath('wammu')))
+        if sys.platform == 'win32':
+            icon.LoadFile(AppIconPath('wammu'), wx.BITMAP_TYPE_ICO)
+        else:
+            icon.LoadFile(AppIconPath('wammu'), wx.BITMAP_TYPE_PNG)
         self.SetIcon(icon)
 
         self.CreateStatusBar(2)
@@ -207,16 +214,21 @@ class WammuFrame(wx.Frame):
         self.searchinput = wx.TextCtrl(self.searchpanel, -1)
         self.searchinput.SetToolTipString(_('Enter text to search for, please note that it is treated like regullar expression. Matching is done over all fields.'))
         self.searchpanel.sizer.Add(self.searchinput, 1, wx.LEFT | wx.CENTER | wx.EXPAND)
+        self.searchchoice = wx.CheckBox(self.searchpanel, -1, _('Regexp'))
+        self.searchchoice.SetToolTipString(_('Use regullar expression for searching?'))
+        self.searchchoice.SetValue(self.cfg.Read('/Defaults/SearchRegexp', 'yes') == 'yes')
+        self.searchpanel.sizer.Add(self.searchchoice, 0, wx.LEFT | wx.CENTER | wx.EXPAND)
         self.searchclear = wx.Button(self.searchpanel, -1, _('&Clear'), style = wx.BU_EXACTFIT)
         self.searchpanel.sizer.Add(self.searchclear, 0, wx.LEFT | wx.CENTER | wx.EXPAND)
         self.searchpanel.SetSizer(self.searchpanel.sizer)
         self.rightwin.sizer.Add(self.searchpanel, 0, wx.LEFT | wx.ALL | wx.EXPAND)
 
         wx.EVT_TEXT(self.searchinput, self.searchinput.GetId(), self.OnSearch)
+        wx.EVT_CHECKBOX(self.searchchoice, self.searchchoice.GetId(), self.OnSearch)
         wx.EVT_BUTTON(self, self.searchclear.GetId(), self.ClearSearch)
 
         # item browser
-        self.browser = Wammu.Browser.Browser(self.rightwin, self)
+        self.browser = Wammu.Browser.Browser(self.rightwin, self, self.cfg)
         self.rightwin.sizer.Add(self.browser, 1, wx.EXPAND)
         self.rightwin.SetSizer(self.rightwin.sizer)
 
@@ -227,8 +239,7 @@ class WammuFrame(wx.Frame):
         self.rightsplitter.SplitHorizontally(self.rightwin, self.content, self.cfg.ReadInt('/Main/SplitRight', -200))
 
         # initial content
-        self.content.SetPage('<body><html><font size=+1><b>%s</b></font></html></body>' %
-           (_('Welcome to Wammu %s') % Wammu.__version__))
+        self.content.SetContent('<font size=+1><b>%s</b></font>' % (_('Welcome to Wammu %s') % Wammu.__version__))
 
         # Prepare the menu bar
         self.menuBar = wx.MenuBar()
@@ -427,6 +438,7 @@ class WammuFrame(wx.Frame):
         if (self.cfg.Read('/Wammu/AutoConnect', 'no') == 'yes'):
             self.PhoneConnect()
 
+        self.SetupNumberPrefix()
         self.SetupStatusRefresh()
 
     def OnTimer(self, evt = None):
@@ -436,6 +448,7 @@ class WammuFrame(wx.Frame):
                 b = self.sm.GetBatteryCharge()
                 d = self.sm.GetDateTime()
 
+                # Parse power source
                 power = _('Unknown')
                 if b['ChargeState'] == 'BatteryPowered':
                     power = _('battery')
@@ -445,10 +458,47 @@ class WammuFrame(wx.Frame):
                     power = _('no battery')
                 elif b['ChargeState'] == 'PowerFault':
                     power = _('fault')
-                self.SetStatusText(_('Bat: %d %% (%s), Sig: %d %%, Time: %s') % (b['BatteryPercent'], power, s['SignalPercent'], StrConv(d.strftime('%c'))), 1)
+
+                # Time might be None if it is invalid (eg. 0.0.0000 date)
+                if d is None:
+                    time = _('Unknown')
+                else:
+                    time = StrConv(d.strftime('%c'))
+
+                self.SetStatusText(_('Bat: %(battery_percent)d %% (%(power_source)s), Sig: %(signal_percent)d %%, Time: %(time)s') %
+                    {
+                        'battery_percent':b['BatteryPercent'],
+                        'power_source':power,
+                        'signal_percent':s['SignalPercent'],
+                        'time': time
+                    }, 1)
             except gammu.GSMError:
                 pass
 
+    def SetupNumberPrefix(self):
+        self.prefix = self.cfg.Read('/Wammu/PhonePrefix', 'Auto')
+        if self.prefix == 'Auto':
+            if self.connected:
+                self.prefix = None
+                try:
+                    on = Wammu.Utils.ParseMemoryEntry(self.sm.GetMemory(Location = 1, Type = 'ON'))['Number']
+                    self.prefix = Wammu.Utils.GrabNumberPrefix(on, Wammu.Data.InternationalPrefixes)
+                except gammu.GSMError:
+                    pass
+                if self.prefix is None:
+                    try:
+                        smsc = self.sm.GetSMSC()['Number']
+                        self.prefix = Wammu.Utils.GrabNumberPrefix(smsc, Wammu.Data.InternationalPrefixes)
+                    except gammu.GSMError:
+                        pass
+                if self.prefix is None:
+                    self.prefix = self.cfg.Read('/Wammu/LastPhonePrefix', '')
+                else:
+                    self.cfg.Write('/Wammu/LastPhonePrefix', self.prefix)
+            else:
+                self.prefix = self.cfg.Read('/Wammu/LastPhonePrefix', '')
+        Wammu.Utils.NumberPrefix = self.prefix
+
     def SetupStatusRefresh(self):
         repeat = self.cfg.ReadInt('/Wammu/RefreshState', 5000)
         if repeat == 0:
@@ -564,7 +614,13 @@ class WammuFrame(wx.Frame):
         self.ClearSearch()
 
     def OnSearch(self, event):
-        self.browser.Filter(self.searchinput.GetValue())
+        text = self.searchinput.GetValue()
+        type = self.searchchoice.GetValue()
+        try:
+            self.browser.Filter(text, type)
+            self.searchinput.SetBackgroundColour(wx.NullColour)
+        except Wammu.Browser.FilterException:
+            self.searchinput.SetBackgroundColour(wx.RED)
 
     def ClearSearch(self, event = None):
         self.searchinput.SetValue('')
@@ -594,6 +650,7 @@ class WammuFrame(wx.Frame):
                         _('Notice'),
                         wx.OK | wx.ICON_INFORMATION).ShowModal()
             self.DoDebug(self.cfg.Read('/Debug/Show', 'no'))
+            self.SetupNumberPrefix()
             self.SetupStatusRefresh()
 
     def CloseWindow(self, event):
@@ -602,6 +659,10 @@ class WammuFrame(wx.Frame):
             self.SaveWinSize(self.logwin, '/Debug')
         self.cfg.WriteInt('/Main/Split', self.splitter.GetSashPosition())
         self.cfg.WriteInt('/Main/SplitRight', self.rightsplitter.GetSashPosition())
+        if self.searchchoice.GetValue():
+            self.cfg.Write('/Defaults/SearchRegexp', 'yes')
+        else:
+            self.cfg.Write('/Defaults/SearchRegexp', 'no')
 
         gammu.SetDebugFile(None)
         gammu.SetDebugLevel('nothing')
@@ -631,13 +692,14 @@ class WammuFrame(wx.Frame):
                         wx.PD_CAN_ABORT | wx.PD_APP_MODAL | wx.PD_AUTO_HIDE | wx.PD_ELAPSED_TIME | wx.PD_REMAINING_TIME | wx.PD_ESTIMATED_TIME)
 
     def OnProgress(self, evt):
-        if not self.progress.Update(evt.progress):
-            try:
-                evt.cancel()
-            except:
-                pass
-        if (evt.progress == 100):
-            del self.progress
+        if hasattr(self, 'progress'):
+            if not self.progress.Update(evt.progress):
+                try:
+                    evt.cancel()
+                except:
+                    pass
+            if (evt.progress == 100):
+                del self.progress
         if hasattr(evt, 'lock'):
             evt.lock.release()
 
@@ -666,10 +728,7 @@ class WammuFrame(wx.Frame):
                     text += u'<b>%s</b>: %s<br>' % (d[0], d[1])
                 else:
                     text += u'<br>%s' % d[0]
-        if wx.USE_UNICODE:
-            self.content.SetPage(u'<html><head><meta http-equiv="Content-Type" content="text/html; charset=ucs-2"></head><body>%s</body></html>' % StrConv(text))
-        else:
-            self.content.SetPage(HtmlStrConv('<html><head><meta http-equiv="Content-Type" content="text/html; charset=%s"></head><body>%s</body></html>' % (Wammu.Utils.htmlcharset, text)))
+        self.content.SetContent(text)
 
     def OnShow(self, evt):
         data = v = evt.data
@@ -812,7 +871,10 @@ class WammuFrame(wx.Frame):
                     v['Location'] = self.sm.SetMemory(v)
 
                 # reread entry (it doesn't have to contain exactly same data as entered, it depends on phone features)
-                v = self.sm.GetMemory(v['MemoryType'], v['Location'])
+                try:
+                    v = self.sm.GetMemory(v['MemoryType'], v['Location'])
+                except (gammu.ERR_NOTSUPPORTED, gammu.ERR_NOTIMPLEMENTED):
+                    wx.MessageDialog(self, _('It was not possible to read saved entry! It might be different than one saved in phone untill you reread all entries.'), _('Could not read saved entry!'), wx.OK | wx.ICON_WARNING).ShowModal()
                 Wammu.Utils.ParseMemoryEntry(v)
                 v['Synced'] = True
                 # append new value to list
@@ -861,7 +923,10 @@ class WammuFrame(wx.Frame):
                     v['Location'] = self.sm.SetCalendar(v)
 
                 # reread entry (it doesn't have to contain exactly same data as entered, it depends on phone features)
-                v = self.sm.GetCalendar(v['Location'])
+                try:
+                    v = self.sm.GetCalendar(v['Location'])
+                except (gammu.ERR_NOTSUPPORTED, gammu.ERR_NOTIMPLEMENTED):
+                    wx.MessageDialog(self, _('It was not possible to read saved entry! It might be different than one saved in phone untill you reread all entries.'), _('Could not read saved entry!'), wx.OK | wx.ICON_WARNING).ShowModal()
                 Wammu.Utils.ParseCalendar(v)
                 v['Synced'] = True
                 # append new value to list
@@ -903,7 +968,10 @@ class WammuFrame(wx.Frame):
                     v['Location'] = self.sm.SetToDo(v)
 
                 # reread entry (it doesn't have to contain exactly same data as entered, it depends on phone features)
-                v = self.sm.GetToDo(v['Location'])
+                try:
+                    v = self.sm.GetToDo(v['Location'])
+                except (gammu.ERR_NOTSUPPORTED, gammu.ERR_NOTIMPLEMENTED):
+                    wx.MessageDialog(self, _('It was not possible to read saved entry! It might be different than one saved in phone untill you reread all entries.'), _('Could not read saved entry!'), wx.OK | wx.ICON_WARNING).ShowModal()
                 Wammu.Utils.ParseTodo(v)
                 v['Synced'] = True
                 # append new value to list
@@ -1057,7 +1125,7 @@ class WammuFrame(wx.Frame):
 
             self.progress.Update(100)
             del self.progress
-            self.SetStatusText(_('%d messages exported to mailbox "%s"') % (count, path))
+            self.SetStatusText(_('%(count)d messages exported to "%(path)s" (%(type)s)') % {'count':count, 'path':path, 'type': _('mailbox')})
 
         # Maildir
         elif idx == 1:
@@ -1132,7 +1200,7 @@ class WammuFrame(wx.Frame):
             self.progress.Update(100)
             del self.progress
 
-            self.SetStatusText(_('%d messages exported to maildir "%s"') % (count, path))
+            self.SetStatusText(_('%(count)d messages exported to "%(path)s" (%(type)s)') % {'count':count, 'path':path, 'type': _('maildir')})
 
         # IMAP
         elif idx == 2:
@@ -1162,7 +1230,7 @@ class WammuFrame(wx.Frame):
             self.cfg.Write('/IMAP/Login', login)
 
             dlg = wx.PasswordEntryDialog(self,
-                _('Please enter password for %s@%s') % (login, server),
+                _('Please enter password for %(login)s@%(server)s') % {'login': login,'server': server},
                 _('Password'),
                 '')
             if dlg.ShowModal() == wx.ID_CANCEL:
@@ -1243,7 +1311,7 @@ class WammuFrame(wx.Frame):
                 filename, data = Wammu.MailWriter.SMSToMail(self.cfg, sms, self.values['contact']['ME'] + self.values['contact']['SM'])
 
                 try:
-                    res = m.append(folder, '(\Unseen)', None, data)
+                    res = m.append(folder, None, None, data)
                 except:
                     res = ['FAIL']
                 if res[0] != 'OK':
@@ -1251,6 +1319,7 @@ class WammuFrame(wx.Frame):
                         _('Can not save message to folder %s on server, bailing out.') % folder,
                         _('Saving failed!'),
                         wx.OK | wx.ICON_ERROR).ShowModal()
+                    self.progress.Update(100)
                     del self.progress
                     self.SetStatusText(_('Export terminated'))
                     return
@@ -1263,7 +1332,7 @@ class WammuFrame(wx.Frame):
             except:
                 pass
 
-            self.SetStatusText(_('%d messages exported to IMAP server "%s"') % (count, server))
+            self.SetStatusText(_('%(count)d messages exported to "%(path)s" (%(type)s)') % {'count':count, 'path':path, 'type': _('IMAP server')})
 
         else:
             raise Exception('Not implemented export functionality!')
@@ -1272,23 +1341,29 @@ class WammuFrame(wx.Frame):
         wildcard = ''
         if type == 'message':
             wildcard +=  _('Gammu messages backup') + ' (*.smsbackup)|*.smsbackup|'
+            exts = ['smsbackup']
         else:
             if not save:
                 wildcard += _('All backup formats') + '|*.backup;*.lmb;*.vcf;*.ldif;*.vcs;*.ics|'
 
             wildcard +=  _('Gammu backup [all data]') + ' (*.backup)|*.backup|'
+            exts = ['backup']
 
-            if not data:
-                if type in ['contact', 'all']:
-                    wildcard += _('Nokia backup [contacts]') + ' (*.lmb)|*.lmb|'
-                if type in ['contact', 'all']:
-                    wildcard += _('vCard [contacts]') + ' (*.vcf)|*.vcf|'
-                if type in ['contact', 'all']:
-                    wildcard += _('LDIF [concacts]') + ' (*.ldif)|*.ldif|'
-                if type in ['todo', 'calendar', 'all']:
-                    wildcard += _('vCalendar [todo,calendar]') + ' (*.vcs)|*.vcs|'
-                if type in ['todo', 'calendar', 'all']:
-                    wildcard += _('iCalendar [todo,calendar]') + ' (*.ics)|*.ics|'
+            if type in ['contact', 'all']:
+                wildcard += _('Nokia backup [contacts]') + ' (*.lmb)|*.lmb|'
+                exts += ['lmb']
+            if type in ['contact', 'all']:
+                wildcard += _('vCard [contacts]') + ' (*.vcf)|*.vcf|'
+                exts += ['vcf']
+            if type in ['contact', 'all']:
+                wildcard += _('LDIF [concacts]') + ' (*.ldif)|*.ldif|'
+                exts += ['ldif']
+            if type in ['todo', 'calendar', 'all']:
+                wildcard += _('vCalendar [todo,calendar]') + ' (*.vcs)|*.vcs|'
+                exts += ['vcs']
+            if type in ['todo', 'calendar', 'all']:
+                wildcard += _('iCalendar [todo,calendar]') + ' (*.ics)|*.ics|'
+                exts += ['ics']
 
         wildcard += _('All files') + ' (*.*)|*.*;*'
 
@@ -1304,6 +1379,10 @@ class WammuFrame(wx.Frame):
                 dlg = wx.FileDialog(self, _('Import backup'), os.getcwd(), "", wildcard, wx.OPEN|wx.CHANGE_DIR)
         if dlg.ShowModal() == wx.ID_OK:
             path = dlg.GetPath()
+            if save:
+                ext = exts[dlg.GetFilterIndex()]
+                if os.path.splitext(path)[1] == '':
+                    path += '.' + ext
             return path
         return None
 
@@ -1869,6 +1948,7 @@ class WammuFrame(wx.Frame):
         try:
             self.sm.Init()
             self.TogglePhoneMenus(True)
+            self.SetupNumberPrefix()
             try:
                 self.IMEI = self.sm.GetIMEI()
             except:
@@ -1923,8 +2003,11 @@ class WammuFrame(wx.Frame):
         self.PhoneDisconnect()
         d = Wammu.PhoneSearch.LogDialog(self)
         self.searchlog = d
-        t = Wammu.PhoneSearch.AllSearchThread(lock =
-        self.cfg.Read('/Gammu/LockDevice', 'no'), callback = self.SearchDone, msgcallback = self.SearchMessage, win = self)
+        t = Wammu.PhoneSearch.AllSearchThread(
+            lock = self.cfg.Read('/Gammu/LockDevice', 'no'),
+            callback = self.SearchDone,
+            msgcallback = self.SearchMessage,
+            win = self)
         t.start()
         d.ShowModal()
 
@@ -1937,7 +2020,13 @@ class WammuFrame(wx.Frame):
 
         choices = []
         for x in self.founddevices:
-            choices.append(_('Model %s (%s) on %s port using connection %s') % (x[2][1], x[3], x[0], x[1]))
+            choices.append(_('Model %(model)s (%(manufacturer)s) on %(port)s port using connection %(connection)s') %
+                {
+                    'model':x[2][1],
+                    'manufacturer':x[3],
+                    'port':x[0],
+                    'connection': x[1]
+                })
         dlg = wx.SingleChoiceDialog(self, _('Select phone to use from bellow list'), _('Select phone'),
                                     choices, wx.CHOICEDLG_STYLE | wx.RESIZE_BORDER)
         if dlg.ShowModal() == wx.ID_OK:
