@@ -1,22 +1,26 @@
 # -*- coding: UTF-8 -*-
-# Wammu - Phone manager
-# Copyright (c) 2003 - 2006 Michal Čihař
-#
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published by
-# the Free Software Foundation.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
-# more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 '''
+Wammu - Phone manager
 Misc functions like charset conversion, entries parsers,..
 '''
+__author__ = 'Michal Čihař'
+__email__ = 'michal@cihar.com'
+__license__ = '''
+Copyright (c) 2003 - 2006 Michal Čihař
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License version 2 as published by
+the Free Software Foundation.
+
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+'''
 
 import wx
 import codecs
@@ -177,12 +181,7 @@ def SearchLocation(lst, loc, second = None):
                 break
     return result
 
-def MatchesText(item, text):
-    dig = text.isdigit()
-    if dig:
-        num = int(text)
-    match = re.compile(text, re.I)
-
+def MatchesText(item, match, num):
     for x in item:
         if type(item) == dict:
             val = item[x]
@@ -191,7 +190,7 @@ def MatchesText(item, text):
         if type(val) in (str, unicode):
             if match.search(val) != None:
                 return True
-        elif dig and type(val) == int and num == val:
+        elif num is not None and type(val) == int and num == val:
             return True
         elif type(val) == list:
             for i in range(len(val)):
@@ -199,7 +198,7 @@ def MatchesText(item, text):
                 if type(val2) in (str, unicode):
                     if match.search(val2) != None:
                         return True
-                elif dig and type(val2) == int and num == val2:
+                elif num is not None and type(val2) == int and num == val2:
                     return True
     return False
 
@@ -210,10 +209,38 @@ def SearchItem(lst, item):
             return i
     return -1
 
+def GrabNumberPrefix(number, prefixes):
+    l = len(number)
+    if l == 0 or number[0] != '+':
+        return None
+    i = 2
+    while not number[:i] in prefixes:
+        i += 1
+        if i > l:
+            return None
+    return number[:i]
+
+'''Prefix for making international numbers'''
+NumberPrefix = ''
+
+NumberStrip = re.compile('^([#*]\d+[#*])?(\\+?.*)$')
+
+def NormalizeNumber(number):
+    # Strip magic prefixes (like no CLIR)
+    nbmatch = NumberStrip.match(number)
+    resnumber = nbmatch.group(2)
+    # If we stripped whole number, return original
+    if len(resnumber) == 0:
+        return number
+    # Add international prefix
+    if resnumber[0] != '+':
+        resnumber = NumberPrefix + resnumber
+    return resnumber
+
 def SearchNumber(lst, number):
     for i in range(len(lst)):
         for x in lst[i]['Entries']:
-            if GetItemType(x['Type']) == 'phone' and number == x['Value']:
+            if GetItemType(x['Type']) == 'phone' and NormalizeNumber(number) == NormalizeNumber(x['Value']):
                 return i
     return -1
 
@@ -400,5 +427,12 @@ def FormatError(txt, info):
     elif info['Code'] == gammu.Errors['ERR_TIMEOUT']:
         message = _('Timeout while trying to communicate with phone. Maybe phone is not connected (for cable) or out of range (for bluetooth or IrDA).')
     else:
-        message = '%s %s\n%s %s\n%s %d' % (_('Description:'), info['Text'], _('Function:'), info['Where'], _('Error code:'), info['Code'])
+        message = '%s %s\n%s %s\n%s %d' % (_('Description:'), StrConv(info['Text']), _('Function:'), info['Where'], _('Error code:'), info['Code'])
     return StrConv(txt + '\n\n' + message)
+
+def FixupMaskedEdit(edit):
+        # XXX: this is not clean way of reseting to system colour, but I don't know better.
+        bgc = wx.SystemSettings.GetColour(wx.SYS_COLOUR_LISTBOX)
+        fgc = wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOWTEXT)
+        setattr(edit, '_validBackgroundColour', bgc)
+        setattr(edit, '_foregroundColour', fgc)
